<?php
/**
 * A database wrapper for the User class
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, this is free software
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class UserDatabaseWrapper extends BaseDatabaseWrapper {
	// Constants for exceptions
	const EXCEPTION_CLIENT_USERNAME_NOT_FOUND = 0x180;

	// Constants for database columns
	const DB_COLUMN_USERNAME = "username";
	const DB_COLUMN_EMAIL    = "email";

	// Constants for database table names
	const DB_TABLE_USER = "user";

	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct() {
		// Call parent constructor
		parent::__construct(__CLASS__);

		// Set part description
		$this->setObjectDescription("Database wrapper for user objects");

		// Create unique ID number
		$this->generateUniqueId();
	}

	/**
	 * Creates an instance of this database wrapper by a provided user class
	 *
	 * @return	$wrapperInstance	An instance of the created wrapper class
	 * @throws	WrapperUserNameNotFoundException	If the supplied username
	 *												does not exist
	 */
	public final static function createUserDatabaseWrapper () {
		// Get a new instance
		$wrapperInstance = new UserDatabaseWrapper();

		// Set (primary!) table name
		$wrapperInstance->setTableName(self::DB_TABLE_USER);

		// Return the instance
		return $wrapperInstance;
	}

	/**
	 * Handles inserting the registration data from a registration instance into the database
	 *
	 * @param	$registrationInstance	An instance of a registration class
	 * @return	void
	 */
	public function insertRegistrationObject (UserRegister $registrationInstance) {
		// Generate a data set for the request
		$dataSetInstance = ObjectFactory::createObjectByConfiguredName('dataset_criteria_class', array(self::DB_TABLE_USER));

		// Set the primary key
		$dataSetInstance->setUniqueKey('username');

		// Add registration elements to the dataset
		$registrationInstance->addElementsToDataSet($dataSetInstance);

		// "Insert" this request instance completely into the database
		$this->getDatabaseInstance()->queryInsertDataSet($dataSetInstance);
	}

	/**
	 * Updates an user database entry with given result
	 *
	 * @param	$resultInstance		An instance of a Updateable database result
	 * @return	void
	 */
	public function doUpdateByResult (UpdateableResult $resultInstance) {
		// Generate a data set object
		$dataSetInstance = ObjectFactory::createObjectByConfiguredName('dataset_criteria_class', array(self::DB_TABLE_USER));

		// Add all update criteria to the database set
		$resultInstance->addElementsToDataSet($dataSetInstance);

		// Add seach criteria
		$dataSetInstance->setSearchInstance($resultInstance->getSearchInstance());

		// Set the primary key
		$dataSetInstance->setUniqueKey('username');

		// "Update" this request with the database
		$this->getDatabaseInstance()->queryUpdateDataSet($dataSetInstance);
	}
}

// [EOF]
?>
